
# This chart only creates a kinesis stream
resource "aws_kinesis_stream" "stream" {
  name             = var.stream_name
  shard_count      = var.shard_count
  retention_period = var.retention_period


  # shard_level_metrics = [
  #   "IncomingBytes",
  #   "OutgoingBytes",
  # ]

  enforce_consumer_deletion = false

  encryption_type = var.encryption_enabled ? "KMS" : "NONE"
  kms_key_id      = "alias/aws/kinesis"

  tags = var.tags

  # lifecyle policy can be overwridden by kinesis_lifecycle_override.tf
  # lifecycle {}
}


resource "aws_iam_policy" "consumer-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-kinesis-consumer"
  path        = "/"
  description = "Kinesis stream consumer policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "kinesis:Get*",
        "kinesis:Describe*",
        "kinesis:List*",
        "kinesis:RegisterStreamConsumer",
        "kinesis:SubscribeToShard"
      ],
      "Effect": "Allow",
      "Resource": [
        "${aws_kinesis_stream.stream.arn}",
        "${aws_kinesis_stream.stream.arn}/consumer/*"
      ]
    },
    {
      "Action": [
        "cloudwatch:PutMetricData"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }

  tags = var.tags
}

resource "aws_iam_policy" "producer-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-kinesis-producer"
  path        = "/"
  description = "Kinesis stream producer policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "kinesis:DescribeStream",
        "kinesis:DescribeStreamSummary",
        "kinesis:PutRecord",
        "kinesis:PutRecords",
        "kinesis:Get*",
        "kinesis:List*"
      ],
      "Effect": "Allow",
      "Resource": "${aws_kinesis_stream.stream.arn}"
    },
    {
      "Action": [
        "cloudwatch:PutMetricData"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }

  tags = var.tags
}

resource "aws_iam_policy" "shard-management-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-kinesis-shard-mgt"
  path        = "/"
  description = "Kinesis stream shard management"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "kinesis:DescribeStream",
        "kinesis:DescribeStreamSummary",
        "kinesis:SplitShard",
        "kinesis:MergeShards",
        "kinesis:Get*",
        "kinesis:List*"
      ],
      "Effect": "Allow",
      "Resource": "${aws_kinesis_stream.stream.arn}"
    },
    {
      "Action": [
        "cloudwatch:PutMetricData"
      ],
      "Effect": "Allow",
      "Resource": "*"
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }

  tags = var.tags
}
